@@ -29,14 +29,10 @@
 #include <sys/socket.h>
 #include <sys/utsname.h>
 #include <sched.h>
-#include <stdbool.h>
-#include <sys/vfs.h>
 
 #define FUSE_COMMFD_ENV		"_FUSE_COMMFD"
 
-#define FUSE_DEV_OLD "/proc/fs/fuse/dev"
-#define FUSE_DEV_NEW "/dev/fuse"
-#define FUSE_VERSION_FILE_OLD "/proc/fs/fuse/version"
+#define FUSE_DEV "/dev/fuse"
 #define FUSE_CONF "/etc/fuse.conf"
 
 #ifndef MS_DIRSYNC
@@ -213,10 +209,10 @@ static int may_unmount(const char *mnt, int quiet)
 }
 
 /*
- * Check whether the file specified in "fusermount -u" is really a
+ * Check whether the file specified in "fusermount3 -u" is really a
  * mountpoint and not a symlink.  This is necessary otherwise the user
  * could move the mountpoint away and replace it with a symlink
- * pointing to an arbitrary mount, thereby tricking fusermount into
+ * pointing to an arbitrary mount, thereby tricking fusermount3 into
  * unmounting that (umount(2) will follow symlinks).
  *
  * This is the child process running in a separate mount namespace, so
@@ -493,7 +489,7 @@ static int count_fuse_fs(void)
 
 
 #else /* IGNORE_MTAB */
-static int count_fuse_fs()
+static int count_fuse_fs(void)
 {
 	return 0;
 }
@@ -510,6 +506,7 @@ static int add_mount(const char *source, const char *mnt, const char *type,
 
 static int unmount_fuse(const char *mnt, int quiet, int lazy)
 {
+	(void) quiet;
 	return fuse_mnt_umount(progname, mnt, mnt, lazy);
 }
 #endif /* IGNORE_MTAB */
@@ -567,19 +564,10 @@ static void read_conf(void)
 			fprintf(stderr, "%s: reading %s: missing newline at end of file\n", progname, FUSE_CONF);
 
 		}
-		if (ferror(fp)) {
-			fprintf(stderr, "%s: reading %s: read failed\n", progname, FUSE_CONF);
-			exit(1);
-		}
 		fclose(fp);
 	} else if (errno != ENOENT) {
-		bool fatal = (errno != EACCES && errno != ELOOP &&
-			      errno != ENAMETOOLONG && errno != ENOTDIR &&
-			      errno != EOVERFLOW);
 		fprintf(stderr, "%s: failed to open %s: %s\n",
 			progname, FUSE_CONF, strerror(errno));
-		if (fatal)
-			exit(1);
 	}
 }
 
@@ -725,7 +713,7 @@ static int get_string_opt(const char *s, unsigned len, const char *opt,
 
 static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 		    int fd, const char *opts, const char *dev, char **sourcep,
-		    char **mnt_optsp, off_t rootsize)
+		    char **mnt_optsp)
 {
 	int res;
 	int flags = MS_NOSUID | MS_NODEV;
@@ -737,7 +725,6 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 	char *subtype = NULL;
 	char *source = NULL;
 	char *type = NULL;
-	int check_empty = 1;
 	int blkdev = 0;
 
 	optbuf = (char *) malloc(strlen(opts) + 128);
@@ -750,10 +737,8 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 		unsigned len;
 		const char *fsname_str = "fsname=";
 		const char *subtype_str = "subtype=";
-		bool escape_ok = begins_with(s, fsname_str) ||
-				 begins_with(s, subtype_str);
 		for (len = 0; s[len]; len++) {
-			if (escape_ok && s[len] == '\\' && s[len + 1])
+			if (s[len] == '\\' && s[len + 1])
 				len++;
 			else if (s[len] == ',')
 				break;
@@ -772,8 +757,6 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 				goto err;
 			}
 			blkdev = 1;
-		} else if (opt_eq(s, len, "nonempty")) {
-			check_empty = 0;
 		} else if (opt_eq(s, len, "auto_unmount")) {
 			auto_unmount = 1;
 		} else if (!begins_with(s, "fd=") &&
@@ -798,7 +781,7 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 			if (getuid() != 0 && !user_allow_other &&
 			    (opt_eq(s, len, "allow_other") ||
 			     opt_eq(s, len, "allow_root"))) {
-				fprintf(stderr, "%s: option %.*s only allowed if 'user_allow_other' is set in /etc/fuse.conf\n", progname, len, s);
+				fprintf(stderr, "%s: option %.*s only allowed if 'user_allow_other' is set in %s\n", progname, len, s, FUSE_CONF);
 				goto err;
 			}
 			if (!skip_option) {
@@ -807,16 +790,10 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 						flags |= flag;
 					else
 						flags  &= ~flag;
-				} else if (opt_eq(s, len, "default_permissions") ||
-					   opt_eq(s, len, "allow_other") ||
-					   begins_with(s, "max_read=") ||
-					   begins_with(s, "blksize=")) {
+				} else {
 					memcpy(d, s, len);
 					d += len;
 					*d++ = ',';
-				} else {
-					fprintf(stderr, "%s: unknown option '%.*s'\n", progname, len, s);
-					exit(1);
 				}
 			}
 		}
@@ -832,10 +809,6 @@ static int do_mount(const char *mnt, char **typep, mode_t rootmode,
 	sprintf(d, "fd=%i,rootmode=%o,user_id=%u,group_id=%u",
 		fd, rootmode, getuid(), getgid());
 
-	if (check_empty &&
-	    fuse_mnt_check_empty(progname, mnt, rootmode, rootsize) == -1)
-		goto err;
-
 	source = malloc((fsname ? strlen(fsname) : 0) +
 			(subtype ? strlen(subtype) : 0) + strlen(dev) + 32);
 
@@ -902,44 +875,11 @@ err:
 	return -1;
 }
 
-static int check_version(const char *dev)
-{
-	int res;
-	int majorver;
-	int minorver;
-	const char *version_file;
-	FILE *vf;
-
-	if (strcmp(dev, FUSE_DEV_OLD) != 0)
-		return 0;
-
-	version_file = FUSE_VERSION_FILE_OLD;
-	vf = fopen(version_file, "r");
-	if (vf == NULL) {
-		fprintf(stderr, "%s: kernel interface too old\n", progname);
-		return -1;
-	}
-	res = fscanf(vf, "%i.%i", &majorver, &minorver);
-	fclose(vf);
-	if (res != 2) {
-		fprintf(stderr, "%s: error reading %s\n", progname,
-			version_file);
-		return -1;
-	}
-	if (majorver < 3) {
-		fprintf(stderr, "%s: kernel interface too old\n", progname);
-		return -1;
-	}
-	return 0;
-}
-
 static int check_perm(const char **mntp, struct stat *stbuf, int *mountpoint_fd)
 {
 	int res;
 	const char *mnt = *mntp;
 	const char *origmnt = mnt;
-	struct statfs fs_buf;
-	size_t i;
 
 	res = lstat(mnt, stbuf);
 	if (res == -1) {
@@ -1012,54 +952,8 @@ static int check_perm(const char **mntp, struct stat *stbuf, int *mountpoint_fd)
 		return -1;
 	}
 
-	/* Do not permit mounting over anything in procfs - it has a couple
-	 * places to which we have "write access" without being supposed to be
-	 * able to just put anything we want there.
-	 * Luckily, without allow_other, we can't get other users to actually
-	 * use any fake information we try to put there anyway.
-	 * Use a whitelist to be safe. */
-	if (statfs(*mntp, &fs_buf)) {
-		fprintf(stderr, "%s: failed to access mountpoint %s: %s\n",
-			progname, mnt, strerror(errno));
-		return -1;
-	}
 
-	/* Use the same list of permitted filesystems for the mount target as
-	 * the ecryptfs mount helper
-	 * (https://bazaar.launchpad.net/~ecryptfs/ecryptfs/trunk/view/head:/src/utils/mount.ecryptfs_private.c#L225). */
-	typeof(fs_buf.f_type) f_type_whitelist[] = {
-		0x5346414f /* OPENAFS_SUPER_MAGIC */,
-		0x61756673 /* AUFS_SUPER_MAGIC */,
-		0x9123683E /* BTRFS_SUPER_MAGIC */,
-		0x00C36400 /* CEPH_SUPER_MAGIC */,
-		0xFF534D42 /* CIFS_MAGIC_NUMBER */,
-		0x0000F15F /* ECRYPTFS_SUPER_MAGIC */,
-		0x0000EF53 /* EXT[234]_SUPER_MAGIC */,
-		0xF2F52010 /* F2FS_SUPER_MAGIC */,
-		0x65735546 /* FUSE_SUPER_MAGIC */,
-		0x01161970 /* GFS2_MAGIC */,
-		0x3153464A /* JFS_SUPER_MAGIC */,
-		0x000072B6 /* JFFS2_SUPER_MAGIC */,
-		0x0000564C /* NCP_SUPER_MAGIC */,
-		0x00006969 /* NFS_SUPER_MAGIC */,
-		0x00003434 /* NILFS_SUPER_MAGIC */,
-		0x5346544E /* NTFS_SB_MAGIC */,
-		0x794C7630 /* OVERLAYFS_SUPER_MAGIC */,
-		0x52654973 /* REISERFS_SUPER_MAGIC */,
-		0x73717368 /* SQUASHFS_MAGIC */,
-		0x01021994 /* TMPFS_MAGIC */,
-		0x24051905 /* UBIFS_SUPER_MAGIC */,
-		0x58465342 /* XFS_SB_MAGIC */,
-		0x2FC12FC1 /* ZFS_SUPER_MAGIC */,
-	};
-	for (i = 0; i < sizeof(f_type_whitelist)/sizeof(f_type_whitelist[0]); i++) {
-		if (f_type_whitelist[i] == fs_buf.f_type)
-			return 0;
-	}
-
-	fprintf(stderr, "%s: mounting over filesystem type %#010lx is forbidden\n",
-		progname, (unsigned long)fs_buf.f_type);
-	return -1;
+	return 0;
 }
 
 static int try_open(const char *dev, char **devp, int silent)
@@ -1086,20 +980,11 @@ static int try_open(const char *dev, char **devp, int silent)
 static int try_open_fuse_device(char **devp)
 {
 	int fd;
-	int err;
 
 	drop_privs();
-	fd = try_open(FUSE_DEV_NEW, devp, 0);
+	fd = try_open(FUSE_DEV, devp, 0);
 	restore_privs();
-	if (fd >= 0)
-		return fd;
-
-	err = fd;
-	fd = try_open(FUSE_DEV_OLD, devp, 1);
-	if (fd >= 0)
-		return fd;
-
-	return err;
+	return fd;
 }
 
 static int open_fuse_device(char **devp)
@@ -1138,21 +1023,16 @@ static int mount_fuse(const char *mnt, const char *opts)
 	if (getuid() != 0 && mount_max != -1) {
 		int mount_count = count_fuse_fs();
 		if (mount_count >= mount_max) {
-			fprintf(stderr, "%s: too many FUSE filesystems mounted; mount_max=N can be set in /etc/fuse.conf\n", progname);
+			fprintf(stderr, "%s: too many FUSE filesystems mounted; mount_max=N can be set in %s\n", progname, FUSE_CONF);
 			goto fail_close_fd;
 		}
 	}
 
-	res = check_version(dev);
-	if (res != -1) {
-		res = check_perm(&real_mnt, &stbuf, &mountpoint_fd);
-		restore_privs();
-		if (res != -1)
-			res = do_mount(real_mnt, &type, stbuf.st_mode & S_IFMT,
-				       fd, opts, dev, &source, &mnt_opts,
-				       stbuf.st_size);
-	} else
-		restore_privs();
+	res = check_perm(&real_mnt, &stbuf, &mountpoint_fd);
+	restore_privs();
+	if (res != -1)
+		res = do_mount(real_mnt, &type, stbuf.st_mode & S_IFMT,
+			       fd, opts, dev, &source, &mnt_opts);
 
 	if (mountpoint_fd != -1)
 		close(mountpoint_fd);
@@ -1226,22 +1106,21 @@ static int send_fd(int sock_fd, int fd)
 
 static void usage(void)
 {
-	fprintf(stderr,
-		"%s: [options] mountpoint\n"
-		"Options:\n"
-		" -h		    print help\n"
-		" -V		    print version\n"
-		" -o opt[,opt...]   mount options\n"
-		" -u		    unmount\n"
-		" -q		    quiet\n"
-		" -z		    lazy unmount\n",
-		progname);
+	printf("%s: [options] mountpoint\n"
+	       "Options:\n"
+	       " -h		    print help\n"
+	       " -V		    print version\n"
+	       " -o opt[,opt...]   mount options\n"
+	       " -u		    unmount\n"
+	       " -q		    quiet\n"
+	       " -z		    lazy unmount\n",
+	       progname);
 	exit(1);
 }
 
 static void show_version(void)
 {
-	printf("fusermount version: %s\n", PACKAGE_VERSION);
+	printf("fusermount3 version: %s\n", PACKAGE_VERSION);
 	exit(0);
 }
 
@@ -1328,7 +1207,7 @@ int main(int argc, char *argv[])
 		res = chdir("/");
 		if (res == -1) {
 			fprintf(stderr, "%s: failed to chdir to '/'\n", progname);
-			exit(1);
+			goto err_out;
 		}
 	}
 	restore_privs();
@@ -1343,31 +1222,33 @@ int main(int argc, char *argv[])
 	if (commfd == NULL) {
 		fprintf(stderr, "%s: old style mounting not supported\n",
 			progname);
-		exit(1);
+		goto err_out;
 	}
 
 	fd = mount_fuse(mnt, opts);
 	if (fd == -1)
-		exit(1);
+		goto err_out;
 
 	cfd = atoi(commfd);
 	res = send_fd(cfd, fd);
 	if (res == -1)
-		exit(1);
+		goto err_out;
 	close(fd);
 
-	if (!auto_unmount)
+	if (!auto_unmount) {
+		free(mnt);
 		return 0;
+	}
 
 	/* Become a daemon and wait for the parent to exit or die.
-	   ie For the control socket to get closed. 
+	   ie For the control socket to get closed.
 	   btw We don't want to use daemon() function here because
 	   it forks and messes with the file descriptors. */
 	setsid();
 	res = chdir("/");
 	if (res == -1) {
 		fprintf(stderr, "%s: failed to chdir to '/'\n", progname);
-		exit(1);
+		goto err_out;
 	}
 
 	sigfillset(&sigset);
@@ -1400,6 +1281,10 @@ do_unmount:
 				progname, mnt, strerror(errno));
 	}
 	if (res == -1)
-		exit(1);
+		goto err_out;
 	return 0;
+
+err_out:
+	free(mnt);
+	exit(1);
 }
